/*
 * RunATMBuggy.java
 *
 * Created on September 11, 2002, 10:59 PM
 */

package bank;


import java.util.Random;
import IO.RedmondMsgIn;
/**
 *
 * @author  mike
 */
public class RunATMBuggy {
    
    /** Creates a new instance of RunATMBuggy */
    public RunATMBuggy() {
    }
    
    /**
     * Static method to display an array of accounts
     */
    private static void displayAccounts(Account all[]) {
        for (int cnt = 0; cnt < all.length; cnt++) {
            System.out.println(all[cnt]);
        }
        return;
    }
    
    
    /**
     * Static method to find a specified account in an array of accounts
     *
     * Given:
     *         Array of Accounts to search
     *         Account Number and PIN to search for
     * 
     * Returns:
     *         Index of array element with matching Account
     *           (-1 if not found)
     */
    private static int findAccount(Account all[], String acct, String pin) {
        //assert all != null;  // defend against crazy caller
        int indx = -1;  // default to not found
        // check all accounts for match
        for (int cnt = 0; cnt < all.length; cnt++) {
            // determine if matches acct# and pin
            if (all[cnt].matchAcctNum(acct) && all[cnt].getPin().equals(pin)) {
                indx = cnt;
            }
        }
        return indx;
    }
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        ////////////////////////////////////////////////////////////////
        // 1) setup section
        //     create 15 bank accounts with random account # and PIN, zero balance
        ////////////////////////////////////////////////////////////////
        Random generator = new Random();
        Account all[] = new Account[15];  // define array of 10 accounts
        // actually create the accounts
        // Significant Java difference - the array itself is just a reference really
        for (int cnt = 0; cnt < all.length; cnt++) {
            // assign a pin randomly - generated random integer and convert to string
            int rand = generator.nextInt(10000);
            String newPin = Integer.toString(rand);
            all[cnt] = new Account(newPin,0);
        }
        //assert 5 < 3;  // just to show an assert go off
        System.out.println("In this dummy bank there are only 15 valid accounts. Work with these:");
        displayAccounts(all);
        //////////////////////////////////////////////////////////////////
        // 2) Processing section
        //      The processing of customers should continue until a customer 
        //      quits AND then, somebody (presumably a bank worker responds 
        //      that it is time for the system to come down). While the system 
        //      is up, each new customer should be asked for an account number 
        //      and pin, which should be validated. Once a customer has been 
        //      validated, they will be continually given a choice of making a 
        //      deposit, making a withdrawal, doing an account inquiry or 
        //      quitting. The customer will stay active until they say that 
        //      they want to quit. After they quit, ask the administrator if 
        //      the system should stay up  this gives us an opportunity to 
        //      work with as many or as few customers as we desire.
        //////////////////////////////////////////////////////////////////////
        // default to not time to quit
        char answ = 'Y';
        boolean ok;
        int found = -1; // default to not found
        // loop while not time to quit
        while ((answ != 'N') && (answ != 'n')) {
            // ready for a customer
            // ensure we get a valid one
            boolean acctOk = false;
            while (acctOk == false) {
                String acctNum = RedmondMsgIn.readString("Please enter your account number");
                String currPIN = RedmondMsgIn.readString("Please enter your PIN");
                // check to see if matches
                found = findAccount(all, acctNum, currPIN);
                if (found >= 0) { // really found it
                    acctOk = true;    // get out of loop
                }
            }
            // got a good customer - let them at their account
            int choice1 = 0;
            // let them work until they choose quit
            while (choice1 != 4) {
                // with updated version of readValidIntRobust would need to catch UserCancelException
                //choice = RedmondMsgIn.readValidIntRobust("Choose:   1) Deposit 2) Withdraw 3) Account Balance 4) Quit",1,4);
                int choice = RedmondMsgIn.readValidInt("Choose:   1) Deposit 2) Withdraw 3) Account Balance 4) Quit",1,4);
                // process whatever the customer chose to do
                switch (choice) {
                    case 1:                            
                        double depositAmt = -1;   
                        depositAmt = RedmondMsgIn.readValid("Please enter the deposit amount (between 1 and 2000)",1.0,2000.0);
                        ok = all[found].deposit(depositAmt);
                    case 2:
                        double withdrawAmt = -1; 
                        withdrawAmt = RedmondMsgIn.readValid("Please enter the withdrawal amount (between 20 and 200)",20.0,200.0);
                        ok = all[found].withdraw(withdrawAmt);
                        break;
                    case 3:
                        System.out.println("Account: " + all[found]);
                        break;                       
                    default:
                        // didn't choose an action needing a response
                        // (4 will just cause an exit of the loop
                    }  // end switch
                } // end while choice
            answ = RedmondMsgIn.readValidYN("Should the system stay up?");
            
            } // end while answ
        ////////////////////////////////////////////////////////////////
        // 3) final reporting
        //     display the 15 bank accounts 
        ////////////////////////////////////////////////////////////////
        System.out.println("Times up!");
        System.out.println("Updated accounts:");
        displayAccounts(all);

        System.exit(0);
    } // end main
    
} // end class definition
